
import math
import itertools

class GraphRawPair:
    def __init__(self, pair_id, label_1, conf_1, label_2, conf_2, dx, dy, dz, size_ratio, inverted):
        self.pair_id = pair_id

        self.label_1 = label_1
        self.conf_1 = conf_1

        self.label_2 = label_2
        self.conf_2 = conf_2

        self.delta_x = dx
        self.delta_y = dy
        self.delta_z = dz

        self.size_ratio = size_ratio

        self.inverted = inverted

    def __str__(self):
        template = "[Pair({0:s}): \"{1:s}\" ({2:.2f}%) - \"{3:s}\" ({4:.2f}%) " + \
                   "<{5:.4f}, {6:.4f}, {7:.4f}>, D1/D2= {8:.4f}%, {9:s}>]"
        return template.format(self.pair_id, self.label_1, self.conf_1 * 100.0, self.label_2, self.conf_2 * 100.0,
                               self.delta_x, self.delta_y, self.delta_z, self.size_ratio * 100.0,
                               "INV" if self.inverted else "NORM")

    def __repr__(self):
        return self.__str__()

    @staticmethod
    def FromGraphEdge(graph, v_1, top_classes_1, v_2, top_classes_2):
        id_1 = v_1.strID()
        id_2 = v_2.strID()

        # unique pair identifier ...
        pair_id = id_1 + "|" + id_2

        # assume vectors are in cache ...
        dx, dy, dz = graph.cache_3D_vectors[graph.vertices_by_id[id_1]][graph.vertices_by_id[id_2]]

        size_ratio = graph.pairSizeRatio(id_1, id_2)
        inv_size_ratio = 1.0 / size_ratio

        # compute 3D vector ...
        # dx, dy, dz = graph.pairUnitVector3D(id_1, id_2)

        pair_combinations = []
        total_mult_conf = 0.0
        for (label_1, conf_1), (label_2, conf_2) in itertools.product(top_classes_1, top_classes_2):
            # order by labels
            if label_1 <= label_2:
                # keep original order
                na_label, na_conf = (label_1, conf_1)
                nb_label, nb_conf = (label_2, conf_2)
                p_dx, p_dy = dx, dy
                p_ratio = size_ratio
                inverted = False
            else:
                # swap node 1 and 2
                na_label, na_conf = (label_2, conf_2)
                nb_label, nb_conf = (label_1, conf_1)
                p_dx, p_dy = -dx, -dy
                p_ratio = inv_size_ratio
                inverted = True

            pair = GraphRawPair(pair_id, na_label, na_conf, nb_label, nb_conf, p_dx, p_dy, dz, p_ratio, inverted)
            pair_combinations.append(pair)

            total_mult_conf += conf_1 * conf_2

        return pair_combinations, total_mult_conf

    @staticmethod
    def InvertPair(other):
        copy = GraphRawPair(other.pair_id, other.label_2, other.conf_2, other.label_1, other.conf_1,
                            -other.delta_x, -other.delta_y, other.delta_z, 1.0 / other.size_ratio, not other.inverted)

        return copy