
import numpy as np

class GraphMatchGroup:
    def __init__(self, aggregator_value=None):
        self.aggregator_value = aggregator_value
        self.matches_per_image = {}
        self.regions_per_image = {}

        # group_scores[high_idx], high_idx, match_group
        self.score_cache_values = []
        self.score_cache_indices = []

    def add_match(self, image_id, match, region=None):
        self.matches_per_image[image_id] = match

        if region is None:
            region = match.candidate_region()

        self.regions_per_image[image_id] = region

        self.score_cache_indices.append(image_id)
        self.score_cache_values.append(match.total_score)

    def max_score(self):
        idx = np.argmax(self.score_cache_values)
        image_idx = self.score_cache_indices[idx]

        return image_idx, self.score_cache_values[idx]

    def last_active_region(self):
        return self.regions_per_image[max(self.score_cache_indices)]

    def last_added_region(self):
        return self.regions_per_image[self.score_cache_indices[-1]]



    @staticmethod
    def FromGroup(matches_per_image, regions_per_image=None, aggregator_value=None):
        # create empty group ...
        match_group = GraphMatchGroup(aggregator_value)

        if regions_per_image is None:
            regions_per_image = {}

        # add regions ...
        for image_idx in matches_per_image:
            if image_idx in regions_per_image:
                match_region = regions_per_image[image_idx]
            else:
                match_region = None

            match_group.add_match(image_idx, matches_per_image[image_idx], match_region)

        return match_group

